@@ -3,6 +3,7 @@ package DBIx::SearchBuilder::Record;
 
 use strict;
 use vars qw($VERSION @ISA $AUTOLOAD);
+use Class::ReturnValue;
 
 
 $VERSION = '$VERSION$';
@@ -369,12 +370,16 @@ sub id  {
 
 # }}}
 
+=head2 primary_keys
 =head2 PrimaryKeys
 
 Matt Knopp owes docs for this function.
 
 =cut
 
+
+
+*primary_keys = \&PrimaryKeys;
 sub PrimaryKeys { 
     my $self = shift; 
     my %hash = map { $_ => $self->{'values'}->{$_} } @{$self->{'_PrimaryKeys'}};
@@ -459,6 +464,7 @@ sub AUTOLOAD  {
 
 # {{{ sub _Accessible 
 
+*_accessible = \&Accessible;
 sub _Accessible  {
   my $self = shift;
   my $attr = shift;
@@ -488,6 +494,8 @@ sub _Accessible  {
 
 =cut
 
+
+*_accessible_load = \&AccessibleLoad;
 sub _AccessibleLoad {
   my $self = shift;
   while ( my $col = shift ) {
@@ -526,6 +534,7 @@ overrid __Value.
 
 =cut
 
+*__value = \&__Value;
 sub __Value {
  my $self = shift;
  my $field = shift;
@@ -546,6 +555,7 @@ Subclasses can override _Value to insert custom access control.
 
 =cut
 
+*_value = \&_Value;
 sub _Value  {
   my $self = shift;
   return ($self->__Value(@_));
@@ -563,6 +573,7 @@ Subclasses can override _Set to insert custom access control.
 
 =cut
 
+*_set = \&_Set;
 sub _Set {
     my $self = shift;
     return ($self->__Set(@_));
@@ -570,55 +581,79 @@ sub _Set {
 
 
 
-sub __Set  {
-  my $self = shift;
+*__set = \&__Set;
+sub __Set {
+    my $self = shift;
 
-  my %args = ( 'Field' => undef,
-	       'Value' => undef,
-	       'IsSQL' => undef,
-	       @_ );
+    my %args = ( 'Field' => undef,
+                 'Value' => undef,
+                 'IsSQL' => undef,
+                 @_ );
 
-  $args{'Column'} = $args{'Field'}; 
-  $args{'IsSqlFunction'} = $args{'IsSQL'}; 
- 
-  ## Cleanup the hash.
-  delete $args{'Field'};
-  delete $args{'IsSQL'};
+    $args{'Column'}        = $args{'Field'};
+    $args{'IsSQLFunction'} = $args{'IsSQL'};
 
-  
-  if (defined $args{'Column'}) {
-      my $column = lc $args{'Column'};
-      if ((defined $self->__Value($column))  and
-	  ($args{'Value'} eq $self->__Value($column))) {
-	  return (0, "That is already the current value");
-      } 
-      elsif (!defined ($args{'Value'})) {
-	  return (0,"No value sent to _Set!\n");
-      } 
-      else {
+    my $ret = Class::ReturnValue->new();
 
-	  my $method = "Validate".$args{'Column'};
-	  unless ($self->$method($args{'Value'})) {
-	      return(0, 'Illegal value for '.$args{'Column'});
-	  }
+    ## Cleanup the hash.
+    delete $args{'Field'};
+    delete $args{'IsSQL'};
 
-          $args{'Table'}        = $self->Table();
-          $args{'PrimaryKeys'} = {$self->PrimaryKeys()};
-                       
+    unless ( defined( $args{'Column'} ) && $args{'Column'} ) {
+        $ret->as_array( 0, 'No column specified' );
+        $ret->as_error( errno   => 5,
+                            message => "No column specified" );
+        return ($ret->return_value);
+    }
+    my $column = lc $args{'Column'};
+    if (     ( defined $self->__Value($column) )
+         and ( $args{'Value'} eq $self->__Value($column) ) ) {
+        $ret->as_array( 0, "That is already the current value" );
+        $ret->as_error( errno   => 1,
+                            message => "That is already the current value" );
+        return ($ret->return_value);
+    }
+    elsif ( !defined( $args{'Value'} ) ) {
+        $ret->as_array( 0, "No value passed to _Set" );
+        $ret->as_error( errno   => 2,
+                            message => "No value passed to _Set" );
+        return ($ret->return_value);
+    }
+    else {
 
-          my $val = $self->_Handle->UpdateRecordValue(%args);
-	  unless ($val) {
-		return(0, $args{'Column'}." could not be set to ".
-		        $args{'Value'}. ".");
-	  }
+        my $method = "Validate" . $args{'Column'};
+        unless ( $self->$method( $args{'Value'} ) ) {
+            $ret->as_array( 0, 'Illegal value for ' . $args{'Column'} );
+            $ret->as_error(errno   => 3,
+                               message => "Illegal value for " . $args{'Column'}
+            );
+            return ($ret->return_value);
+        }
 
-	  $self->{'values'}->{"$column"} = $args{'Value'};
-      }
-      return (1, "The new value has been set.");
-  }
-  else {
-      return(0, 'No column specified');
-  }
+        $args{'Table'}       = $self->Table();
+        $args{'PrimaryKeys'} = { $self->PrimaryKeys() };
+
+        my $val = $self->_Handle->UpdateRecordValue(%args);
+        unless ($val) {
+            $ret->as_array( 0,
+                                $args{'Column'}
+                                  . " could not be set to "
+                                  . $args{'Value'} . "." );
+            $ret->as_error( errno   => 4,
+                                message => $args{'Column'}
+                                  . " could not be set to "
+                                  . $args{'Value'} . "." );
+            return ($ret->return_value);
+        }
+        if ( $args{'IsSQLFunction'} ) {
+            $self->Load( $self->Id );
+        }
+        else {
+            $self->{'values'}->{"$column"} = $args{'Value'};
+        }
+    }
+    $ret->as_array( 1, "The new value has been set." );
+    return ($ret->return_value);
 }
 
 # }}}
@@ -626,6 +661,8 @@ sub __Set  {
 # {{{ sub _Validate 
 
 #TODO: Implement _Validate.
+
+*_validate = \&_Validate;
 sub _Validate  {
     my $self = shift;
     my $field = shift;
@@ -664,6 +701,7 @@ is $id
 =cut
 
 
+*load = \&Load;
 sub Load  {
     my $self = shift;
     my ($package, $filename, $line) = caller;
@@ -681,6 +719,8 @@ undefined
 
 =cut
 
+*load_by_col = \&LoadByCol;
+
 sub LoadByCol  {
     my $self = shift;
     my $col = shift;
@@ -701,6 +741,7 @@ keys.
 
 =cut
 
+*load_by_cols = \&LoadByCols;
 sub LoadByCols  {
     my $self = shift;
     my %hash  = (@_);
@@ -732,6 +773,7 @@ TODO: BUG: Column name is currently hard coded to 'id'
 
 =cut
 
+*load_by_id = \&LoadById;
 sub LoadById  {
     my $self = shift;
     my $id = shift;
@@ -742,6 +784,7 @@ sub LoadById  {
 
 # }}}  
 
+*load_by_primary_keys = \&LoadByPrimaryKeys;
 sub LoadByPrimaryKeys {
     my ($self, $data) = @_;
 
@@ -768,6 +811,8 @@ loaded values hash.
 
 =cut
 
+*load_from_hash = \&LoadFromHash;
+
 sub LoadFromHash {
   my $self = shift;
   my $hashref = shift;
@@ -781,40 +826,88 @@ sub LoadFromHash {
 
 # {{{ sub _LoadFromSQL 
 
-sub _LoadFromSQL  {
-    my $self = shift;
+*load_from_sql = \&LoadFromSQL;
+
+
+sub _LoadFromSQL {
+    my $self        = shift;
     my $QueryString = shift;
     my @bind_values = (@_);
-    
-    my $sth = $self->_Handle->SimpleQuery($QueryString, @bind_values);
-        
+
+    my $sth = $self->_Handle->SimpleQuery( $QueryString, @bind_values );
+
     #TODO this only gets the first row. we should check if there are more.
 
-    eval {
-	$self->{'values'} = $sth->fetchrow_hashref;
-    };
-    if ($@){ 
-	warn $@;
+
+    unless ($sth) {
+        return($sth);
     }
-    
-    unless ($self->{'values'}) {
-	#warn "something might be wrong here; row not found. SQL: $QueryString";
-	return (0, "Couldn't find row");
+
+    eval { $self->{'values'} = $sth->fetchrow_hashref; };
+    if ($@) {
+        warn $@;
     }
-    
+
+    unless ( $self->{'values'} ) {
+
+        #warn "something might be wrong here; row not found. SQL: $QueryString";
+        return ( 0, "Couldn't find row" );
+    }
+
     $self->_DowncaseValuesHash();
-    
-     ## I guess to be consistant with the old code, make sure the primary  
+
+    ## I guess to be consistant with the old code, make sure the primary  
     ## keys exist.
-    
-    eval { 
-	$self->PrimaryKeys();
-    }; if ($@) { 
-	return (0, "Missing a primary key?: $@");
+
+    eval { $self->PrimaryKeys(); };
+    if ($@) {
+        return ( 0, "Missing a primary key?: $@" );
+    }
+    return ( 1, "Found Object" );
+
+}
+
+sub _LoadFromSQLold {
+    my $self        = shift;
+    my $QueryString = shift;
+    my @bind_values = (@_);
+
+    my $sth = $self->_Handle->SimpleQuery( $QueryString, @bind_values );
+
+    #TODO this only gets the first row. we should check if there are more.
+
+
+    return($sth) unless ($sth) ;
+
+    my $fetched;
+    eval { $fetched = $sth->fetchrow_hashref() };
+    if ($@) {
+        warn $@;
+    }
+
+    unless ( $fetched ) {
+
+        #warn "something might be wrong here; row not found. SQL: $QueryString";
+        return ( 0, "Couldn't find row" );
     }
-    return (1, "Found Object");
     
-  }
+    $self->{'values'} = $fetched;
+
+    ## I guess to be consistant with the old code, make sure the primary  
+    ## keys exist.
+      $self->_DowncaseValuesHash();
+    
+        ## I guess to be consistant with the old code, make sure the primary  
+        ## keys exist.
+    
+      eval { $self->PrimaryKeys(); };
+      if ($@) {
+          return ( 0, "Missing a primary key?: $@" );
+      }
+      return ( 1, "Found Object" );
+
+
+}
 
 # }}}
 
@@ -831,6 +924,8 @@ as columns for this recordtype
 
 =cut 
 
+*create = \&Create;
+
 sub Create  {
     my $self = shift;
     my %attribs = @_;
@@ -849,6 +944,9 @@ sub Create  {
 
 # {{{ sub Delete 
 
+*delete =  \&Delete;
+
+
 sub Delete  {
     my $self = shift;
     
@@ -882,6 +980,8 @@ Returns or sets the name of the current Table
 
 =cut
 
+*table = \&Table;
+
 sub Table {
     my $self = shift;
     if (@_) {
@@ -900,6 +1000,8 @@ sub Table {
 Returns or sets the current DBIx::SearchBuilder::Handle object
 
 =cut
+
+*_handle = \&_Handle;
 sub _Handle  {
     my $self = shift;
     if (@_) {
@@ -923,6 +1025,8 @@ sure that all keys are lowercase.
 
 =cut
 
+*_downcase_values_hash = \&DowncaseValuesHash;
+
 sub _DowncaseValuesHash {
     my $self = shift;
     my ($key);
@@ -932,6 +1036,7 @@ sub _DowncaseValuesHash {
     }
     
     $self->{'values'} = $self->{'new_values'};
+    delete $self->{'new_values'};
 }
 
 # }}}
